==> diffsptk-3.0.2-py3-none-any.whl: diffsptk/modules/csm2acr.py <==
# ------------------------------------------------------------------------ #
# Copyright 2022 SPTK Working Group                                        #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License");          #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#     http://www.apache.org/licenses/LICENSE-2.0                           #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
# ------------------------------------------------------------------------ #

import torch

from ..typing import Precomputed
from ..utils.private import check_size, get_values, to
from .base import BaseFunctionalModule


class CompositeSinusoidalModelCoefficientsToAutocorrelation(BaseFunctionalModule):
    """See `this page <https://sp-nitech.github.io/sptk/latest/main/csm2acr.html>`_
    for details.

    Parameters
    ----------
    acr_order : int >= 0
        The order of the autocorrelation, :math:`M`.

    References
    ----------
    .. [1] S. Sagayama et al., "Duality theory of composite sinusoidal modeling and
           linear prediction," *Proceedings of ICASSP*, pp. 1261-1264, 1986.

    """

    def __init__(self, acr_order: int) -> None:
        super().__init__()

        self.in_dim = acr_order + 1

        _, _, tensors = self._precompute(*get_values(locals()))
        self.register_buffer("ramp", tensors[0])

    def forward(self, c: torch.Tensor) -> torch.Tensor:
        """Convert CSM coefficients to autocorrelation.

        Parameters
        ----------
        c : Tensor [shape=(..., M+1)]
            The CSM coefficients.

        Returns
        -------
        out : Tensor [shape=(..., M+1)]
            The autocorrelation.

        Examples
        --------
        >>> x = diffsptk.nrand(4)
        >>> acorr = diffsptk.Autocorrelation(5, 3)
        >>> acr2csm = diffsptk.AutocorrelationToCompositeSinusoidalModelCoefficients(3)
        >>> csm2acr = diffsptk.CompositeSinusoidalModelCoefficientsToAutocorrelation(3)
        >>> r = acorr(x)
        >>> r
        tensor([ 8.8894, -0.1102, -4.1748,  0.7501])
        >>> r2 = csm2acr(acr2csm(r))
        >>> r2
        tensor([ 8.8894, -0.1102, -4.1748,  0.7501])

        """
        check_size(c.size(-1), self.in_dim, "dimension of autocorrelation")
        return self._forward(c, **self._buffers)

    @staticmethod
    def _func(c: torch.Tensor, *args, **kwargs) -> torch.Tensor:
        _, _, tensors = (
            CompositeSinusoidalModelCoefficientsToAutocorrelation._precompute(
                c.size(-1) - 1, *args, **kwargs, device=c.device, dtype=c.dtype
            )
        )
        return CompositeSinusoidalModelCoefficientsToAutocorrelation._forward(
            c, *tensors
        )

    @staticmethod
    def _takes_input_size() -> bool:
        return True

    @staticmethod
    def _check(acr_order: int) -> None:
        if acr_order <= 0 or acr_order % 2 == 0:
            raise ValueError("acr_order must be a positive odd number.")

    @staticmethod
    def _precompute(
        acr_order: int,
        device: torch.device | None = None,
        dtype: torch.dtype | None = None,
    ) -> Precomputed:
        CompositeSinusoidalModelCoefficientsToAutocorrelation._check(acr_order)
        ramp = torch.arange(acr_order + 1, device=device)
        return None, None, (to(ramp, dtype=dtype),)

    @staticmethod
    def _forward(c: torch.Tensor, ramp: torch.Tensor) -> torch.Tensor:
        w, m = torch.tensor_split(c, 2, dim=-1)
        a = m.unsqueeze(-2)  # (..., 1, (M+1)/2)
        b = torch.cos(w.unsqueeze(-1) * ramp)  # (..., (M+1)/2, M+1)
        r = torch.matmul(a, b).squeeze(-2)  # (..., M+1)
        return r

==> diffsptk-3.3.0-py3-none-any.whl: diffsptk/modules/csm2acr.py <==
# ------------------------------------------------------------------------ #
# Copyright 2022 SPTK Working Group                                        #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License");          #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#     http://www.apache.org/licenses/LICENSE-2.0                           #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
# ------------------------------------------------------------------------ #

import torch

from ..typing import Precomputed
from ..utils.private import check_size, filter_values, to
from .base import BaseFunctionalModule


class CompositeSinusoidalModelCoefficientsToAutocorrelation(BaseFunctionalModule):
    """See `this page <https://sp-nitech.github.io/sptk/latest/main/csm2acr.html>`_
    for details.

    Parameters
    ----------
    acr_order : int >= 0
        The order of the autocorrelation, :math:`M`.

    device : torch.device or None
        The device of this module.

    dtype : torch.dtype or None
        The data type of this module.

    References
    ----------
    .. [1] S. Sagayama et al., "Duality theory of composite sinusoidal modeling and
           linear prediction," *Proceedings of ICASSP*, pp. 1261-1264, 1986.

    """

    def __init__(
        self,
        acr_order: int,
        device: torch.device | None = None,
        dtype: torch.dtype | None = None,
    ) -> None:
        super().__init__()

        self.in_dim = acr_order + 1

        _, _, tensors = self._precompute(**filter_values(locals()))
        self.register_buffer("ramp", tensors[0])

    def forward(self, c: torch.Tensor) -> torch.Tensor:
        """Convert CSM coefficients to autocorrelation.

        Parameters
        ----------
        c : Tensor [shape=(..., M+1)]
            The CSM coefficients.

        Returns
        -------
        out : Tensor [shape=(..., M+1)]
            The autocorrelation.

        Examples
        --------
        >>> x = diffsptk.nrand(4)
        >>> acorr = diffsptk.Autocorrelation(5, 3)
        >>> acr2csm = diffsptk.AutocorrelationToCompositeSinusoidalModelCoefficients(3)
        >>> csm2acr = diffsptk.CompositeSinusoidalModelCoefficientsToAutocorrelation(3)
        >>> r = acorr(x)
        >>> r
        tensor([ 8.8894, -0.1102, -4.1748,  0.7501])
        >>> r2 = csm2acr(acr2csm(r))
        >>> r2
        tensor([ 8.8894, -0.1102, -4.1748,  0.7501])

        """
        check_size(c.size(-1), self.in_dim, "dimension of autocorrelation")
        return self._forward(c, **self._buffers)

    @staticmethod
    def _func(c: torch.Tensor, *args, **kwargs) -> torch.Tensor:
        _, _, tensors = (
            CompositeSinusoidalModelCoefficientsToAutocorrelation._precompute(
                c.size(-1) - 1, *args, **kwargs, device=c.device, dtype=c.dtype
            )
        )
        return CompositeSinusoidalModelCoefficientsToAutocorrelation._forward(
            c, *tensors
        )

    @staticmethod
    def _takes_input_size() -> bool:
        return True

    @staticmethod
    def _check(acr_order: int) -> None:
        if acr_order <= 0 or acr_order % 2 == 0:
            raise ValueError("acr_order must be a positive odd number.")

    @staticmethod
    def _precompute(
        acr_order: int,
        device: torch.device | None,
        dtype: torch.dtype | None,
    ) -> Precomputed:
        CompositeSinusoidalModelCoefficientsToAutocorrelation._check(acr_order)
        ramp = torch.arange(acr_order + 1, device=device)
        return None, None, (to(ramp, dtype=dtype),)

    @staticmethod
    def _forward(c: torch.Tensor, ramp: torch.Tensor) -> torch.Tensor:
        w, m = torch.tensor_split(c, 2, dim=-1)
        a = m.unsqueeze(-2)  # (..., 1, (M+1)/2)
        b = torch.cos(w.unsqueeze(-1) * ramp)  # (..., (M+1)/2, M+1)
        r = torch.matmul(a, b).squeeze(-2)  # (..., M+1)
        return r
